#version 120

uniform float u_shadowMaxAlpha, u_shadowGamma;
uniform vec3 u_projectedShadowDrawMin, u_projectedShadowDrawMax;
uniform vec2 u_xyDistanceMultiplier; // Usually 1.0, but can set to 0.0 to make an edge hard

varying vec3 v_worldPosition;
varying float v_lightRadius; // half penumbra width
varying vec2 v_unitTexCoord, v_constCastPlaneSize;
varying vec4 v_color;

float rand(vec2 co)
{
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

void main()
{
    vec2 proportionFromEdge = vec2(0.5) - abs(vec2(0.5) - v_unitTexCoord); // this point distance from nearest corner, normalized so (0.5, 0.5) is center
    vec2 worldDistanceFromEdge = proportionFromEdge * v_constCastPlaneSize; // this point distance from nearest corner, in inches
    vec2 distanceIntoLight = max(vec2(2.0 * v_lightRadius) - worldDistanceFromEdge, vec2(0.0)) * u_xyDistanceMultiplier; // measured from "the edge" of the light shape, how much of it is visible to us at this point ({0, 0} in this case would be "all dark", {2*v_lightRadius,2*v_lightRadius} "all light")
    vec2 proportionIntoLight = distanceIntoLight / (v_lightRadius * 2.0);

    // Crop on the bottom and top of the shelf, if the shelf has planks
    if (any(lessThan(v_worldPosition, u_projectedShadowDrawMin)) ||
        any(greaterThan(v_worldPosition, u_projectedShadowDrawMax)))
        discard;

    // draw the dang shadow
    float lightness = clamp(length(proportionIntoLight), 0, 1);

// WJS: Attempt to lighten corners, like we see when we do real ambient occlusion on rectangular objects. Doesn't actually work very well.
//    float normalizedDistanceFromEdge = length(vec2(1) - proportionIntoLight);
//    const float distanceWhereWeLightenCorners = 1;
//    if (normalizedDistanceFromEdge < distanceWhereWeLightenCorners) {
//        lightness += pow(clamp(distanceWhereWeLightenCorners - normalizedDistanceFromEdge, 0, 1), 2);
//        lightness = clamp(lightness, 0, 1);
//    }

    float blueNoise = (rand(v_worldPosition.xy) - 0.5) * 1/256;
    gl_FragColor = vec4(0, 0, 0, pow(clamp(1 - lightness, 0.0, 1.0), u_shadowGamma) * u_shadowMaxAlpha + blueNoise);
}
